// 003 u_c_http_raw_logs
// 24 dec 2001

// -- (C) John COLIBRI 2001
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_http_raw_logs;
  interface
    uses Classes, u_types_constants
        , u_c_basic_file
        , u_c_direct_access_variable_file
        ;

    // 192.168.1.2, -, 24/08/2001, 18:34:44, W3SVC4, S3, 192.168.1.246, 220, 325, 10825,
    //                                                                             bytes_sent
    //                                                                       bytes_received
    //                                                                  elapsed_time
    //
    //    200, 0, GET, /Default.asp, -,
    //                               params
    //                 target
    //            get_or_post
    //         windows_status_code
    //    status_code
    type t_field_type= (e_client_ip,
                        e_user_name, e_date, e_time, e_service, e_computer_name,
                        e_server_ip, e_elapsed_time, e_bytes_received, e_bytes_sent,
                        e_status_code, e_windows_status_code, e_GET_or_POST,
                        e_target, e_params, e_end);


    type c_http_raw_logs= class(c_basic_file)
                            public
                              // -- the positions in the direct access variable file
                              m_log_record: array[e_client_ip..e_params] of Integer;

                              m_c_direct_access_variable_file: c_direct_access_variable_file;

                              constructor create_http_raw_logs(p_name, p_file_name: String); Virtual;

                              function f_force_create: Boolean; Override;
                              function f_open: Boolean; Override;
                              procedure close_file; Override;

                              procedure append_log(p_ascii_file_name: String);

                              procedure read_record;

                                function f_read_field(p_field_type: t_field_type): String;
                              function f_client_ip: String;
                              function f_date: String;
                              function f_time: String;
                              function f_elapsed_time: String;
                              function f_bytes_received: String;
                              function f_bytes_sent: String;
                              function f_status_code: string;
                              function f_GET_or_POST: String;
                              function f_target: String;
                              function f_params: String;

                              function f_the_day_and_time: String;
                              function f_target_and_params: String;

                              function f_elapsed_time_value: Integer;

                              procedure display_record;

                              destructor Destroy; Override;
                          end;

  implementation
    uses SysUtils, Dialogs
        , u_c_display
        , u_c_text_file
        ;

    function f_field_name(p_field: t_field_type): String;
      begin
        Case p_field of
          e_client_ip: Result:= 'client_ip';
          e_user_name : Result:= 'user_name';
          e_date : Result:= 'date';
          e_time : Result:= 'time';
          e_service : Result:= 'service';
          e_computer_name : Result:= 'computer_name';

          e_server_ip : Result:= 'server_ip';
          e_elapsed_time : Result:= 'elapsed_time';
          e_bytes_received : Result:= 'bytes_received';
          e_bytes_sent : Result:= 'bytes_sent';

          e_status_code : Result:= 'status_code';
          e_windows_status_code : Result:= 'windows_status_code';
          e_GET_or_POST : Result:= 'get_or_post';
          e_target : Result:= 'targe_=t';
          e_params : Result:= 'params';
          e_end : Result:= 'end';
          else Result:= '???';
        end; // case p_field
      end; // f_field_name

    // -- c_http_raw_logs

    constructor c_http_raw_logs.create_http_raw_logs(p_name, p_file_name: String);
      begin
        Inherited create_basic_file(p_name, p_file_name+ '.log');
        m_c_direct_access_variable_file:= c_direct_access_variable_file.create_direct_access_variable_file('', p_file_name);
      end; // create_http_raw_logs

    function c_http_raw_logs.f_force_create: Boolean;
      begin
        Result:= Inherited f_force_create and m_c_direct_access_variable_file.f_force_create;
      end; // f_force_create

    function c_http_raw_logs.f_open: Boolean;
      begin
        m_display_open:= True;
        Result:= Inherited f_open and m_c_direct_access_variable_file.f_open;
      end; // f_open

    procedure c_http_raw_logs.close_file;
      begin
        Inherited;
        m_c_direct_access_variable_file.close_file;
      end; // close_file

    procedure c_http_raw_logs.append_log(p_ascii_file_name: String);
      var l_c_text_file: c_text_file;

      procedure analyze;
        var l_log_array: array[e_client_ip..e_params] of String;

        procedure empty_fields;
          var l_field_1: t_field_type;
          begin
            for l_field_1:= e_client_ip to e_params do
              l_log_array[l_field_1]:= '';
          end; // empty_fields

        procedure save_log_array;
          var l_field_1: t_field_type;
          begin
            // display(l_log_array[e_client_ip]);
            FillChar(m_log_record, SizeOf(m_log_record), 0);

            for l_field_1:= e_client_ip to e_params do
            begin
              // display(f_field_name(l_field_1));
              m_log_record[l_field_1]:=
                m_c_direct_access_variable_file.f_append_string(m_size,
                  Ord(l_field_1), l_log_array[l_field_1]);
            end;

            // -- now write the log record
            seek_file(m_size);
            f_block_write(@ m_log_record, SizeOf(m_log_record));
          end; // save_log_array

        var l_field: t_field_type;
            l_field_value: String;

        begin // analyze
          with l_c_text_file do
          begin
            l_field:= e_client_ip;
            l_field_value:= '';

            while m_buffer_index< m_buffer_size do
            begin
              case m_pt_buffer[m_buffer_index] of
                ' ' : inc(m_buffer_index);
                ',' : begin
                        l_log_array[l_field]:= l_field_value;
                        l_field:= Succ(l_field);
                        l_field_value:= '';
                        inc(m_buffer_index);
                      end;
                k_return : begin
                             save_log_array;

                             Inc(m_buffer_index, 2);
                             l_field:= e_client_ip;
                             l_field_value:= '';
                             empty_fields;
                           end;
                else
                  l_field_value:= l_field_value+ m_pt_buffer[m_buffer_index];
                  inc(m_buffer_index);
              end; // case
            end; // while
          end; // with l_c_text_file
        end; // analyze

      begin // append_log
        l_c_text_file:= c_text_file.create_text_file('ascii_log', p_ascii_file_name);

        with l_c_text_file do
          if f_load
            then begin
                display('size '+ IntToStr(m_buffer_size));
                if not Self.f_open
                  then Exit;

                analyze;

                display('index '+ IntToStr(m_buffer_index));

                Self.close_file;
              end // load ok
            else display('*** could not open '+ m_c_file_name.m_complete_name
                + ' '+ IntToStr(m_io_result));
      end; // append_log

    procedure c_http_raw_logs.read_record;
      begin
        f_block_read(@ m_log_record, SizeOf(m_log_record));
      end; // read_record

    function c_http_raw_logs.f_read_field(p_field_type: t_field_type): String;
      begin
        with m_c_direct_access_variable_file do
        begin
          seek_file(m_log_record[p_field_type]);
          Result:= f_read_string;
        end;
      end; // f_read_field

    function c_http_raw_logs.f_client_ip: String;
      begin
        Result:= f_read_field(e_client_ip);
      end; // f_client_ip

    function c_http_raw_logs.f_date: String;
      begin
        Result:= f_read_field(e_date);
      end; // f_date

    function c_http_raw_logs.f_time: String;
      begin
        Result:= f_read_field(e_time);
      end; // f_time

    function c_http_raw_logs.f_elapsed_time: String;
      begin
        Result:= f_read_field(e_elapsed_time);
      end; // f_elapsed_time

    function c_http_raw_logs.f_bytes_received: String;
      begin
        Result:= f_read_field(e_bytes_received);
      end; // f_bytes_received

    function c_http_raw_logs.f_bytes_sent: String;
      begin
        Result:= f_read_field(e_bytes_sent);
      end; // f_bytes_sent

    function c_http_raw_logs.f_status_code: string;
      begin
        Result:= f_read_field(e_status_code);
      end; // f_status_code

    function c_http_raw_logs.f_GET_or_POST: String;
      begin
        Result:= f_read_field(e_get_or_post);
      end; // f_GET_or_POST

    function c_http_raw_logs.f_target: String;
      begin
        Result:= f_read_field(e_target);
      end; // f_target

    function c_http_raw_logs.f_params: String;
      begin
        Result:= f_read_field(e_params);
      end; // f_params

    // -- computed fields

    function c_http_raw_logs.f_the_day_and_time: String;
      var l_date, l_time: String;
      begin
        l_date:= f_date; l_time:= f_time;
        Result:= Copy(l_date, 1, 2)+ ' '+ Copy(l_time, 1, 5);
      end; // f_the_day_and_time

    function c_http_raw_logs.f_elapsed_time_value: Integer;
      var l_elapsed_time: String;
      begin
        l_elapsed_time:= f_elapsed_time;
        if l_elapsed_time= ''
          then Result:= 0
          else Result:= StrToInt(l_elapsed_time);
      end; // f_elapsed_time_value

    // -- a example of display

    function c_http_raw_logs.f_target_and_params: String;

      function f_extract_file(p_params: String): String;
        var l_index, l_params_length: Integer;
        begin
          // -- a=10?filename=u_c_basic_object.zip
          l_params_length:= Length(p_params);
          l_index:= l_params_length;
          while (l_index>= 1) and (p_params[l_index]<> '=') do
            Dec(l_index);
          Result:= Copy(p_params, l_index+ 1, l_params_length+ 1- l_index- 1);
        end; // f_extract_file

      function f_the_target(p_target: String): String;
        var l_index, l_target_length: Integer;
        begin
          if Pos('404', p_target)> 0
            then begin
                Result:= '*** '+ p_target;
              end
            else
              begin
                // -- /xxx/yyy/zzz.html: get last
                l_target_length:= Length(p_target);
                l_index:= l_target_length;
                while (l_index>= 1) and (p_target[l_index]<> '/') do
                  Dec(l_index);
                Result:= Copy(p_target, l_index+ 1, l_target_length+ 1- l_index- 1);
              end;
        end; // f_the_target

      var l_target, l_params: String;

      begin // f_target_and_params
        l_target:= f_target;
        l_params:= f_params;

        if Trim(l_params)= '-'
          then Result:= f_the_target(l_target)
          else Result:= f_the_target(l_target)+ ' '+ l_params;
      end; // f_target_and_params

    procedure c_http_raw_logs.display_record;
      var l_text: String;
          l_target_and_params: String;
      begin
        // 192.168.1.2, -, 24/08/2001, 18:34:44, W3SVC4, S3, 192.168.1.246, 220, 325, 10825,
        //                                                                             bytes_sent
        //                                                                       bytes_received
        //                                                                  elapsed_time
        //
        //    200, 0, GET, /Default.asp, -,
        //                               params
        //                 target
        //            get_or_post
        //         windows_status_code
        //    status_code
        l_text:= Format('%-15s %-9s %6s', [f_client_ip, f_the_day_and_time, f_elapsed_time])
            + Format('%6s %6s %-4s ', [f_bytes_received, f_bytes_sent, f_get_or_post]);

        l_target_and_params:= f_target_and_params;
        l_text:= l_text+ l_target_and_params;

        display(l_text);
      end; // display_record

    destructor c_http_raw_logs.Destroy;
      begin
        m_c_direct_access_variable_file.Free;
        Inherited Destroy;
      end; // Destroy

    begin // u_c_http_raw_logs
    end. // u_c_http_raw_logs


